/**
 * ========================================================
 *  <a href="http://www.openolat.org">
 *  OpenOLAT - Online Learning and Training</a><br>
 *  <p>
 *  Licensed under the Apache License, Version 2.0 (the "License"); <br>
 *  you may not use this file except in compliance with the License.<br>
 *  You may obtain a copy of the License at the
 *  <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 *  <p>
 *  Unless required by applicable law or agreed to in writing,<br>
 *  software distributed under the License is distributed on an "AS IS" BASIS, <br>
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 *  See the License for the specific language governing permissions and <br>
 *  limitations under the License.
 *  <p>
 *  Initial code contributed and copyrighted by<br>
 *  28.04.2014 by frentix GmbH, http://www.frentix.com
 *  Modified and extended<br>
 *  20.11.2015 by daniel.haag<AT>uibk.ac.at
 * ========================================================
 *
 *  UIBK theme JS extensions as jQuery module
 *  based on OpenOLAT theme and using the jquery.navbar.js code for the separate tools in the header.
 *
 *  @dependencies jQuery
 */
+(function($) {

	var ThemeJS = function() {
		// nothing to do
	}

	ThemeJS.prototype.moveElementsToHeader = function() {
		var headerContainer = $("#o_header_container");
		if (!headerContainer.hasClass('o_collapse')) {
			headerContainer.addClass("o_collapse");
			headerContainer.addClass("o_navbar-collapse");
		}

		if ($("#o_uibk_ecampus").size() == 0) {
			headerContainer.append('<div id="o_uibk_ecampus"><h2 id="o_uibk_ecampus-heading">eCampus</h2></div>');
		}

		if ($("#o_logo-link").size() == 0) {
			$("#o_logo").append('<a id="o_logo-link" class="o_clickable" href="https://www.uibk.ac.at" target="_blank" title="eCampus - University of Innsbruck"></a>');
		}

		var dirty = false;

		var tools = $("#o_navbar_container #o_navbar_tools_permanent");
		if (tools.size() > 0) {
			$("#o_logo").after(tools.parent());
			dirty = true;
		}

		var logintools = $("#o_navbar_container .o_navbar-collapse .o_navbar_tools");
		if (logintools.size() > 0) {
			$("#o_logo").after(logintools);
			dirty = true;
		} 

		$("#o_top").insertBefore("#o_header_wrapper");

		if (dirty) {
			/* to allow the responsive navigation to react to our removing the tools */
			$(window).trigger('resize');
		}
	}

	/**
	 * Method to install the theme add-ons. Will be re-executed after each DOM replacement. 
	 * 
	 * @method
	 */
	ThemeJS.prototype.execThemeJS = function() {
		OPOL.themejs.moveElementsToHeader()
		// execute after each dom replacement (navbar might have been changed)
		$(document).on("oo.dom.replacement.after", OPOL.themejs.moveElementsToHeader);
	}
	
	/**
	 * Use the carrousel effect for background images on the login screen based 
	 * on the ooBgCarrousel UIBK jQuery plugin
	 */
	ThemeJS.prototype.initDmzCarrousel = function() {
		this.dmzCarrousel = jQuery().ooBgCarrousel();
		this.dmzCarrousel.initCarrousel({
			query: "#o_body.o_dmz #o_bg",
			images: ['siegel-universitaet-innsbruck.jpg', 'siegel-universitaet-innsbruck.jpg' ],
			shuffle: false,
			shuffleFirst: false,
			durationshow: 5000,
			durationout: 500,
			durationin: 1000,
			easeout : 'ease',
			easein : 'ease'
		});
	}


	var Toolbar = function() {
		this.state = {
			busy: false,
			headerW: 0,
			logoW: 0,
			toolsW: 0,
			toolsDirty: false,
			tools: {
				collapsed: this.isToolsCollapsed(),
				extended: this.isToolsExtended()
			},
			offCanvasWidth : 0
		};

                // get site of menu from css
                var w = $('#o_offcanvas_right').css('width');
                //no width in full screen mode
                if(w) {
                        this.state.offCanvasWidth = parseInt(w.replace(/[^\d.]/g,''));

                        this.initListners();
                        this.calculateWidth();
                        this.optimize();
                }

	}

	Toolbar.prototype.initListners = function() {
                // Optimize navbar when the browser changes size
                $(window).resize($.proxy(this.onResizeCallback,this));

                // Mark tool components dirty when updated in DOM
                $(document).on("oo.nav.tools.modified", $.proxy(function() {
                        this.state.toolsDirty = true;
                        //console.log('tools dirty');
                },this));

                // Optimize when DOM replacement is finished (and marked dirty previously)
                $(document).on("oo.dom.replacement.after", $.proxy(this.onDOMreplacementCallback,this));
	}

	Toolbar.prototype.onResizeCallback = function() {
                if (!this.state.busy) {
                        this.state.busy = true;
                        this.calculateWidth();
                        this.optimize();
                        this.state.busy = false;
                }
        }

	Toolbar.prototype.onDOMreplacementCallback = function() {
                if (!this.state.busy && this.state.toolsDirty) {
                        this.state.busy = true;
                        this.cleanupMoreDropdown();
                        this.calculateWidth();
                        this.optimize();
                        this.state.toolsDirty = false;
                        this.state.busy = false;
                }
        }

        Toolbar.prototype.calculateWidth = function() {
                var el = $('#o_header_container');
                // Get the dimensions of the viewport
                this.state.headerW = el.innerWidth();
                // logo
                this.state.logoW = $('#o_logo-link').outerWidth(true);
                // the tools
                //don't include margin, because left and right margins on this element are negative
                this.state.toolsW = this.getTools().outerWidth(false);

                //console.log('calculateWidth w:' + this.state.navbarW + ' s:'+this.state.sitesW + ' d:'+this.state.tabsW + ' t:'+this.state.toolsW + ' o:'+this.getOverflow() );
        }

	Toolbar.prototype.getOverflow = function(e) {
                // Calculate if more space is used in navbar than available. Get total width and substract feature by feature
                var o = this.state.headerW;
                o -= this.state.toolsW;
                o -= this.state.logoW;
                //element widths can sometimes be off by 1px, so we need a small buffer on o
                o -= 25;

                return -o;
        }

        Toolbar.prototype.optimize = function(e) {
                var o = this.getOverflow();
                var tools = this.getTools();
                var offcanvasRight = this.getOffcanvasRight();

                this.updateState();

                //console.log('optimize o:' + o);
                // Move from toolbar to offcanvas
                while (o > 0 && (!this.state.tools.collapsed)) {
                        if (!this.state.tools.collapsed) {
                                console.log('collapse tools ' + o);
                                this.collapse(tools, offcanvasRight, '.o_navbar_tool:not(#o_navbar_imclient, #o_navbar_search_opener, #o_navbar_my_menu)', 'o_tool_right');
                        }

                        this.calculateWidth();
                        o = this.getOverflow();
                        this.updateState();
                }
                // Move from offcanvas to toolbar
                while (o < 0 && (!this.state.tools.extended)) {
                        if (!this.state.tools.extended) {
                                console.log('uncollapse static ' + o);
                                var toolFit = this.extend(offcanvasRight, tools.children('#o_navbar_imclient, #o_navbar_search_opener, #o_navbar_my_menu').first(), '.o_tool_right', 'o_tool_right', true);
                                if(!toolFit){ break; }
                        }

                        this.calculateWidth();
                        o = this.getOverflow();
                        this.updateState();
                }

                this.checkToolsOrder();
        }

        Toolbar.prototype.updateState = function() {
                this.state.tools.collapsed = this.isToolsCollapsed();
                this.state.tools.extended = this.isToolsExtended();
        }

        Toolbar.prototype.collapse = function(source, dest, selector, addClass) {
                var item = source.find(selector)
                if(item.length){ item = item.last(); }
                if(item.length){
                        console.log('collapsing item '+item.attr('class'));
                        if (addClass) {
				item.addClass(addClass);
			}
                        if (dest) {
                                item.prependTo(dest);
                        }
                }
        }

        Toolbar.prototype.extend = function(source, dest, selector, removeClass, before) {
                var item = source.find(selector);
                if(item.length){ item = item.first(); }
                var itemFit = false;
                if(item.length){
                        if(dest){
                                if(before){
                                        dest.before(item);
                                } else {
                                        item.appendTo(dest);
                                }
                                //if item doesn't fit, revert the change
                                this.calculateWidth();
                                var o = this.getOverflow();
                                if(o > 0){
                                        item.prependTo(source);
                                } else {
//                                      console.log('extending item '+item.attr('class'));
                                        removeClass && item.removeClass(removeClass);
                                        itemFit = true;
                                }
                        }
                }

                return itemFit;
        }

        Toolbar.prototype.checkToolsOrder = function() {
                var tools = this.getTools();
                var help = tools.find('#o_navbar_help');
                var print = tools.find('#o_navbar_print');
                var im = tools.find('#o_navbar_imclient');
                if(im && print){
                        im.after(print);
                }
                if(im && help){
                        im.after(help);
                }
        }

        Toolbar.prototype.getOffcanvasRight = function() {
                return $('#o_offcanvas_right_container .o_navbar-right');
        }

        Toolbar.prototype.getTools = function() {
                return $('#o_navbar_tools_permanent');
        }

        Toolbar.prototype.isToolsCollapsed = function() {
                return !this.getTools().children('.o_navbar_tool').not('#o_navbar_imclient, #o_navbar_search_opener, #o_navbar_my_menu').length;
        }

        Toolbar.prototype.isToolsExtended = function() {
                return !this.getOffcanvasRight().children('.o_tool_right').length;
        }

	//Execute when loading of page has been finished
	$(document).ready(function() {
		OPOL.themejs = new ThemeJS();
		OPOL.themejs.execThemeJS();
		OPOL.themejs.initDmzCarrousel();
		var tbar = $('#o_header_container');
		if (tbar) {
			var toolbar = new Toolbar();
			window.OPOL.uibk_theme_toolbar = toolbar;
		}
	});
	
})(jQuery);
